@@ -1,5 +1,4 @@
 import asyncio
-from asyncio.queues import Queue
 from asyncio.tasks import gather
 
 __all__ = (
@@ -207,6 +206,9 @@ class PubSubReply:
     def __repr__(self):
         return 'PubSubReply(channel=%r, value=%r)' % (self.channel, self.value)
 
+    def __eq__(self, other):
+        return self._channel == other._channel and self._value == other._value
+
 
 class EvalScriptReply:
     """
